#
#  Copyright 2018 Spotify AB.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.

# Generated

VERSION = '1.2.23'



#
#  Copyright 2018 Spotify AB.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.

from uuid import uuid4
from . import VERSION
from marshmallow import Schema, fields, post_load, pre_dump


class LoadingPolicy:
    SOME_CONTENT_PLAYTHROUGH = "someContentPlaythrough"
    ALL_CONTENT_PLAYTHROUGH = "allContentPlaythrough"


class ContentType:
    AUDIO = "com.nativeformate.content.audio"

class Script:
    def __init__(self, name, code):
        self.name = name
        self.code = code


class ScriptSchema(Schema):
    name = fields.String(required=True)
    code = fields.String(required=True)

    @post_load
    def make_script(self, data):
        return Script(**data)


class Edge:
    def __init__(self, source, target, source_port=None, target_port=None, id=None):
        self.id = id or str(uuid4())
        self.source = source
        self.target = target
        self.source_port = source_port
        self.target_port = target_port

class EdgeSchema(Schema):
    id = fields.String(required=True)
    source = fields.String(required=True)
    target = fields.String(required=True)
    source_port = fields.String(data_key='sourcePort')
    target_port = fields.String(data_key='targetPort')
    
    @post_load
    def make_edge(self, data):
        return Edge(**data)


class Command:
    def __init__(self, name, args):
        self.name = name
        self.args = args


class CommandSchema(Schema):
    name = fields.String(required=True)
    args = fields.Dict()

    @post_load
    def make_command(self, data):
        return Command(**data)


class Node:
    def __init__(self, kind, config, params, id=None, loading_policy=LoadingPolicy.ALL_CONTENT_PLAYTHROUGH):
        self.id = id or str(uuid4())
        self.kind = kind
        self.config = config
        self.params = params
        self.loading_policy = loading_policy

    def connect(self, target):
        """Creates an Edge with this Node's id set as the source and the given Node's id set as the edge target"""
        return Edge(self.id, target.id)


class NodeSchema(Schema):
    id = fields.String(required=True)
    kind = fields.String(required=True)
    config = fields.Dict()
    params = fields.Dict(values=fields.List(fields.Nested(CommandSchema)))
    loading_policy = fields.String(data_key='loadingPolicy')

    @post_load
    def make_node(self, data):
        return Node(**data)


class Graph:
    def __init__(self, nodes=None, edges=None, scripts=None, id=None, loading_policy=LoadingPolicy.ALL_CONTENT_PLAYTHROUGH):
        self.id = id or str(uuid4())
        self.nodes = nodes or []
        self.edges = edges or []
        self.scripts = scripts or []
        self.loading_policy = loading_policy


class GraphSchema(Schema):
    id = fields.String(required=True)
    nodes = fields.List(fields.Nested(NodeSchema))
    edges = fields.List(fields.Nested(EdgeSchema))
    scripts = fields.List(fields.Nested(ScriptSchema))
    loading_policy = fields.String(data_key='loadingPolicy')

    @post_load
    def make_graph(self, data):
        return Graph(**data)

    @pre_dump
    def convert_typed_nodes(self, graph):
        def convert(n):
            if hasattr(n, 'to_node'):
                return n.to_node()
            return n

        nodes = list(map(convert, graph.nodes))
        graph.nodes = nodes


class Score:
    def __init__(self, graph, version=VERSION):
        self.graph = graph
        self.version = version


class ScoreSchema(Schema):
    graph = fields.Nested(GraphSchema, required=True)
    version = fields.String(required=True)

    @post_load
    def make_score(self, data):
        return Score(**data)



#
#  Copyright 2018 Spotify AB.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.

def assert_kind(value, kind, kind_name, property_name=None):
    if not isinstance(value, kind):
        raise Exception('expected {} to be a {} kind'.format(property_name or 'value', kind_name))


def assert_list_kind(value, kind, kind_name, property_name=None):
    kind_name = 'list({})'.format(kind_name)
    assert_kind(value, list, kind_name, property_name)
    for v in value:
        assert_kind(v, kind, kind_name, property_name)


def assert_string(value, property_name=None):
    assert_kind(value, str, 'string', property_name)


def assert_int(value, property_name=None):
    assert_kind(value, int, 'int', property_name)


def assert_float(value, property_name=None):
    assert_kind(value, float, 'float', property_name)


def assert_bool(value, property_name=None):
    assert_kind(value, bool, 'bool', property_name)


def assert_time(value, property_name=None):
    assert_kind(value, float, 'time', property_name)


def assert_list_string(value, property_name=None):
    assert_list_kind(value, str, 'string', property_name)


def assert_list_int(value, property_name=None):
    assert_list_kind(value, int, 'int', property_name)


def assert_list_float(value, property_name=None):
    assert_list_kind(value, float, 'float', property_name)


def assert_list_bool(value, property_name=None):
    assert_list_kind(value, bool, 'bool', property_name)


def assert_list_time(value, property_name=None):
    assert_list_kind(value, float, 'time', property_name)



#
#  Copyright 2018 Spotify AB.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.

import setuptools
from nf_grapher import VERSION

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nf_grapher",
    version=VERSION,
    author="The Native Format Team",
    description="A library for building Native Format Scores.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://ghe.spotify.net/NativeFormat/nf-grapher",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=['marshmallow==3.0.0b11'],
    python_requires=">=3.3",
)

